@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -49,4 +48,7 @@
 /* Coroutine errors */
 #define FLB_ERR_CORO_STACK_SIZE      -600
 
+/* Reloading */
+#define FLB_ERR_RELOADING_IN_PROGRESS 700
+
 #endif
