@@ -84,6 +84,8 @@ extern std::vector<std::tuple<ares_evsys_t, int>>       evsys_families;
 extern std::vector<std::pair<int, bool>>                families_modes;
 extern std::vector<std::tuple<ares_evsys_t, int, bool>> evsys_families_modes;
 
+// Hopefully a more accurate sleep than sleep_for()
+void                    ares_sleep_time(unsigned int ms);
 
 // Process all pending work on ares-owned file descriptors, plus
 // optionally the given set-of-FDs + work function.
@@ -218,7 +220,6 @@ protected:
   ares_channel_t *channel_;
 };
 
-
 // Mock DNS server to allow responses to be scripted by tests.
 class MockServer {
 public:
@@ -234,7 +235,7 @@ public:
   void SetReplyData(const std::vector<byte> &reply)
   {
     exact_reply_ = reply;
-    reply_ = nullptr;
+    reply_       = nullptr;
   }
 
   void SetReply(const DNSPacket *reply)
@@ -249,7 +250,7 @@ public:
   void SetReplyExpRequest(const DNSPacket *reply, const std::string &request)
   {
     expected_request_ = request;
-    reply_ = reply;
+    reply_            = reply;
   }
 
   void SetReplyQID(int qid)
@@ -289,8 +290,9 @@ public:
 
 private:
   void           ProcessRequest(ares_socket_t fd, struct sockaddr_storage *addr,
-                                ares_socklen_t addrlen, const std::string &reqstr,
-                                int qid, const char *name, int rrtype);
+                                ares_socklen_t addrlen, const std::vector<byte> &req,
+                                const std::string &reqstr, int qid, const char *name,
+                                int rrtype);
   void           ProcessPacket(ares_socket_t fd, struct sockaddr_storage *addr,
                                ares_socklen_t addrlen, byte *data, int len);
   unsigned short udpport_;
@@ -309,7 +311,7 @@ private:
 // Test fixture that uses a mock DNS server.
 class MockChannelOptsTest : public LibraryTest {
 public:
-  MockChannelOptsTest(int count, int family, bool force_tcp,
+  MockChannelOptsTest(int count, int family, bool force_tcp, bool honor_sysconfig,
                       struct ares_options *givenopts, int optmask);
   ~MockChannelOptsTest();
 
@@ -339,7 +341,7 @@ class MockChannelTest
     public ::testing::WithParamInterface<std::pair<int, bool>> {
 public:
   MockChannelTest()
-    : MockChannelOptsTest(1, GetParam().first, GetParam().second, nullptr, 0)
+    : MockChannelOptsTest(1, GetParam().first, GetParam().second, false, nullptr, 0)
   {
   }
 };
@@ -347,7 +349,7 @@ public:
 class MockUDPChannelTest : public MockChannelOptsTest,
                            public ::testing::WithParamInterface<int> {
 public:
-  MockUDPChannelTest() : MockChannelOptsTest(1, GetParam(), false, nullptr, 0)
+  MockUDPChannelTest() : MockChannelOptsTest(1, GetParam(), false, false, nullptr, 0)
   {
   }
 };
@@ -355,7 +357,7 @@ public:
 class MockTCPChannelTest : public MockChannelOptsTest,
                            public ::testing::WithParamInterface<int> {
 public:
-  MockTCPChannelTest() : MockChannelOptsTest(1, GetParam(), true, nullptr, 0)
+  MockTCPChannelTest() : MockChannelOptsTest(1, GetParam(), true, false, nullptr, 0)
   {
   }
 };
@@ -365,7 +367,7 @@ public:
   MockEventThreadOptsTest(int count, ares_evsys_t evsys, int family,
                           bool force_tcp, struct ares_options *givenopts,
                           int optmask)
-    : MockChannelOptsTest(count, family, force_tcp,
+    : MockChannelOptsTest(count, family, force_tcp, false,
                           FillOptionsET(&evopts_, givenopts, evsys),
                           optmask | ARES_OPT_EVENT_THREAD)
   {
@@ -389,6 +391,7 @@ public:
   }
 
   void Process(unsigned int cancel_ms = 0);
+
 private:
   struct ares_options evopts_;
 };
@@ -494,6 +497,51 @@ struct HostResult {
 
 std::ostream &operator<<(std::ostream &os, const HostResult &result);
 
+// C++ wrapper for ares_dns_record_t.
+struct AresDnsRecord {
+  ~AresDnsRecord()
+  {
+    ares_dns_record_destroy(dnsrec_);
+    dnsrec_ = NULL;
+  }
+
+  AresDnsRecord() : dnsrec_(NULL)
+  {
+  }
+
+  void SetDnsRecord(const ares_dns_record_t *dnsrec)
+  {
+    if (dnsrec_ != NULL) {
+      ares_dns_record_destroy(dnsrec_);
+    }
+    if (dnsrec == NULL) {
+      return;
+    }
+    dnsrec_ = ares_dns_record_duplicate(dnsrec);
+  }
+
+  ares_dns_record_t *dnsrec_ = NULL;
+};
+
+std::ostream &operator<<(std::ostream &os, const AresDnsRecord &result);
+
+// Structure that describes the result of an ares_host_callback invocation.
+struct QueryResult {
+  QueryResult() : done_(false), status_(ARES_SUCCESS), timeouts_(0)
+  {
+  }
+
+  // Whether the callback has been invoked.
+  bool          done_;
+  // Explicitly provided result information.
+  ares_status_t status_;
+  size_t        timeouts_;
+  // Contents of the ares_dns_record_t structure if provided
+  AresDnsRecord dnsrec_;
+};
+
+std::ostream &operator<<(std::ostream &os, const QueryResult &result);
+
 // Structure that describes the result of an ares_callback invocation.
 struct SearchResult {
   // Whether the callback has been invoked.
@@ -554,6 +602,8 @@ std::ostream &operator<<(std::ostream &os, const AddrInfoResult &result);
 // structures.
 void          HostCallback(void *data, int status, int timeouts,
                            struct hostent *hostent);
+void          QueryCallback(void *data, ares_status_t status, size_t timeouts,
+                            const ares_dns_record_t *dnsrec);
 void SearchCallback(void *data, int status, int timeouts, unsigned char *abuf,
                     int alen);
 void SearchCallbackDnsRec(void *data, ares_status_t status, size_t timeouts,
@@ -714,6 +764,50 @@ int RunInContainer(ContainerFilesystem *fs, const std::string &hostname,
     }                                                                         \
     int ICLASS_NAME(casename, testname)::InnerTestBody()
 
+#define CONTAINED_TEST_P(test_suite_name, test_name, hostname, domainname, files) \
+  class GTEST_TEST_CLASS_NAME_(test_suite_name, test_name)                     \
+      : public test_suite_name {                                               \
+   public:                                                                     \
+    GTEST_TEST_CLASS_NAME_(test_suite_name, test_name)() {}                    \
+    int InnerTestBody();                                                       \
+    void TestBody()                                                            \
+    {                                                                          \
+      ContainerFilesystem chroot(files, "..");                                 \
+      VoidToIntFn         fn =                                                 \
+        [this](void) -> int {                                                  \
+          ares_reinit(this->channel_);                                         \
+          ares_sleep_time(100);                                                \
+          return this->InnerTestBody();                                        \
+        };                                                                     \
+      EXPECT_EQ(0, RunInContainer(&chroot, hostname, domainname, fn));         \
+    }                                                                          \
+                                                                               \
+   private:                                                                    \
+    static int AddToRegistry() {                                               \
+      ::testing::UnitTest::GetInstance()                                       \
+          ->parameterized_test_registry()                                      \
+          .GetTestSuitePatternHolder<test_suite_name>(                         \
+              GTEST_STRINGIFY_(test_suite_name),                               \
+              ::testing::internal::CodeLocation(__FILE__, __LINE__))           \
+          ->AddTestPattern(                                                    \
+              GTEST_STRINGIFY_(test_suite_name), GTEST_STRINGIFY_(test_name),  \
+              new ::testing::internal::TestMetaFactory<GTEST_TEST_CLASS_NAME_( \
+                  test_suite_name, test_name)>(),                              \
+              ::testing::internal::CodeLocation(__FILE__, __LINE__));          \
+      return 0;                                                                \
+    }                                                                          \
+    static int gtest_registering_dummy_ GTEST_ATTRIBUTE_UNUSED_;               \
+    GTEST_TEST_CLASS_NAME_(test_suite_name, test_name)                         \
+    (const GTEST_TEST_CLASS_NAME_(test_suite_name, test_name) &) = delete;     \
+    GTEST_TEST_CLASS_NAME_(test_suite_name, test_name) & operator=(            \
+        const GTEST_TEST_CLASS_NAME_(test_suite_name,                          \
+                                     test_name) &) = delete; /* NOLINT */      \
+  };                                                                           \
+  int GTEST_TEST_CLASS_NAME_(test_suite_name,                                  \
+                             test_name)::gtest_registering_dummy_ =            \
+      GTEST_TEST_CLASS_NAME_(test_suite_name, test_name)::AddToRegistry();     \
+  int GTEST_TEST_CLASS_NAME_(test_suite_name, test_name)::InnerTestBody()
+
 #endif
 
 /* Assigns virtual IO functions to a channel. These functions simply call
