@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -26,10 +25,12 @@
 #include <fluent-bit/flb_mem.h>
 #include <fluent-bit/flb_str.h>
 #include <fluent-bit/flb_filter.h>
-#include <fluent-bit/flb_hash.h>
+#include <fluent-bit/flb_hash_table.h>
 #include <fluent-bit/flb_utils.h>
 #include <fluent-bit/flb_pack.h>
 #include <fluent-bit/flb_filter_plugin.h>
+#include <fluent-bit/flb_log_event_decoder.h>
+#include <fluent-bit/flb_log_event_encoder.h>
 #include <msgpack.h>
 
 #include "geoip2.h"
@@ -41,78 +42,80 @@ static int configure(struct geoip2_ctx *ctx,
     struct mk_list *head = NULL;
     struct mk_list *split;
     int status;
-    struct geoip2_lookup_key *key;
     struct geoip2_record *record;
     struct flb_split_entry *sentry;
+    struct flb_config_map_val *record_key;
+    int ret;
 
     ctx->mmdb = flb_malloc(sizeof(MMDB_s));
     ctx->lookup_keys_num = 0;
     ctx->records_num = 0;
 
-    /* Iterate all filter properties */
-    mk_list_foreach(head, &f_ins->properties) {
-        kv = mk_list_entry(head, struct flb_kv, _head);
+    ret = flb_filter_config_map_set(f_ins, (void *)ctx);
+    if (ret == -1) {
+        flb_plg_error(f_ins, "unable to load configuration");
+        flb_free(ctx->mmdb);
+        return -1;
+    }
 
-        if (strcasecmp(kv->key, "database") == 0) {
-            status = MMDB_open(kv->val, MMDB_MODE_MMAP, ctx->mmdb);
-            if (status != MMDB_SUCCESS) {
-                flb_plg_error(ctx->ins, "Cannot open geoip2 database: %s: %s",
-                              kv->val, MMDB_strerror(status));
-                flb_free(ctx->mmdb);
-                return -1;
-            }
+    if (ctx->database) {
+        status = MMDB_open(ctx->database, MMDB_MODE_MMAP, ctx->mmdb);
+        if (status != MMDB_SUCCESS) {
+            flb_plg_error(f_ins, "Cannot open geoip2 database: %s: %s",
+                          ctx->database, MMDB_strerror(status));
+            flb_free(ctx->mmdb);
+            return -1;
         }
-        else if (strcasecmp(kv->key, "lookup_key") == 0) {
-            key = flb_malloc(sizeof(struct geoip2_lookup_key));
-            if (!key) {
-                flb_errno();
-                continue;
-            }
-            key->key = flb_strndup(kv->val, flb_sds_len(kv->val));
-            key->key_len = flb_sds_len(kv->val);
-            mk_list_add(&key->_head, &ctx->lookup_keys);
-            ctx->lookup_keys_num++;
+    } else {
+        flb_plg_error(f_ins, "no geoip2 database has been loaded");
+        flb_free(ctx->mmdb);
+        return -1;
+    }
+    
+    mk_list_foreach(head, ctx->lookup_keys) {
+        ctx->lookup_keys_num++;
+    }
+    
+    flb_config_map_foreach(head, record_key, ctx->record_keys) {
+        record = flb_malloc(sizeof(struct geoip2_record));
+        if (!record) {
+            flb_errno();
+            continue;
         }
-        else if (strcasecmp(kv->key, "record") == 0) {
-            record = flb_malloc(sizeof(struct geoip2_record));
-            if (!record) {
-                flb_errno();
-                continue;
-            }
-            split = flb_utils_split(kv->val, ' ', 2);
-            if (mk_list_size(split) != 3) {
-                flb_plg_error(ctx->ins, "invalid record parameter: '%s'", kv->val);
-                flb_plg_error(ctx->ins, "expects 'KEY LOOKUP_KEY VALUE'");
-                flb_free(record);
-                flb_utils_split_free(split);
-                continue;
-            }
-            /* Get first value (field) */
-            sentry = mk_list_entry_first(split, struct flb_split_entry, _head);
-            record->key = flb_strndup(sentry->value, sentry->len);
-            record->key_len = sentry->len;
-
-            sentry = mk_list_entry_next(&sentry->_head, struct flb_split_entry,
-                                        _head, split);
-            record->lookup_key = flb_strndup(sentry->value, sentry->len);
-            record->lookup_key_len = sentry->len;
-
-            sentry = mk_list_entry_last(split, struct flb_split_entry, _head);
-            record->val = flb_strndup(sentry->value, sentry->len);
-            record->val_len = sentry->len;
-
+        split = flb_utils_split(record_key->val.str, ' ', 2);
+        if (mk_list_size(split) != 3) {
+            flb_plg_error(f_ins, "invalid record parameter: '%s'", kv->val);
+            flb_plg_error(f_ins, "expects 'KEY LOOKUP_KEY VALUE'");
+            flb_free(record);
             flb_utils_split_free(split);
-            mk_list_add(&record->_head, &ctx->records);
-            ctx->records_num++;
+            continue;
         }
+
+        /* Get first value (field) */
+        sentry = mk_list_entry_first(split, struct flb_split_entry, _head);
+        record->key = flb_strndup(sentry->value, sentry->len);
+        record->key_len = sentry->len;
+
+        sentry = mk_list_entry_next(&sentry->_head, struct flb_split_entry,
+                                    _head, split);
+        record->lookup_key = flb_strndup(sentry->value, sentry->len);
+        record->lookup_key_len = sentry->len;
+
+        sentry = mk_list_entry_last(split, struct flb_split_entry, _head);
+        record->val = flb_strndup(sentry->value, sentry->len);
+        record->val_len = sentry->len;
+
+        flb_utils_split_free(split);
+        mk_list_add(&record->_head, &ctx->records);
+        ctx->records_num++;
     }
 
     if (ctx->lookup_keys_num <= 0) {
-        flb_plg_error(ctx->ins, "lookup_key is required at least one");
+        flb_plg_error(f_ins, "at least one lookup_key is required");
         return -1;
     }
     if (ctx->records_num <= 0) {
-        flb_plg_error(ctx->ins, "record is required at least one");
+        flb_plg_error(f_ins, "at least one record is required");
         return -1;
     }
     return 0;
@@ -122,15 +125,8 @@ static int delete_list(struct geoip2_ctx *ctx)
 {
     struct mk_list *tmp;
     struct mk_list *head;
-    struct geoip2_lookup_key *key;
     struct geoip2_record *record;
 
-    mk_list_foreach_safe(head, tmp, &ctx->lookup_keys) {
-        key = mk_list_entry(head, struct geoip2_lookup_key, _head);
-        flb_free(key->key);
-        mk_list_del(&key->_head);
-        flb_free(key);
-    }
     mk_list_foreach_safe(head, tmp, &ctx->records) {
         record = mk_list_entry(head, struct geoip2_record, _head);
         flb_free(record->lookup_key);
@@ -142,16 +138,20 @@ static int delete_list(struct geoip2_ctx *ctx)
     return 0;
 }
 
-static struct flb_hash *prepare_lookup_keys(msgpack_object *map,
-                                            struct geoip2_ctx *ctx)
+static struct flb_hash_table *prepare_lookup_keys(msgpack_object *map,
+                                                 struct geoip2_ctx *ctx)
 {
     msgpack_object_kv *kv;
     msgpack_object *key;
     msgpack_object *val;
     struct mk_list *head;
-    struct mk_list *tmp;
-    struct geoip2_lookup_key *lookup_key;
-    struct flb_hash *ht = flb_hash_create(FLB_HASH_EVICT_NONE, ctx->lookup_keys_num, -1);
+    struct flb_config_map_val *lookup_key;
+    struct flb_hash_table *ht;
+
+    ht = flb_hash_table_create(FLB_HASH_TABLE_EVICT_NONE, ctx->lookup_keys_num, -1);
+    if (!ht) {
+        return NULL;
+    }
 
     kv = map->via.map.ptr;
     for (int i = 0; i < map->via.map.size; i++) {
@@ -163,11 +163,12 @@ static struct flb_hash *prepare_lookup_keys(msgpack_object *map,
         if (val->type != MSGPACK_OBJECT_STR) {
             continue;
         }
-        mk_list_foreach_safe(head, tmp, &ctx->lookup_keys) {
-            lookup_key = mk_list_entry(head, struct geoip2_lookup_key, _head);
-            if (strncasecmp(key->via.str.ptr, lookup_key->key, lookup_key->key_len) == 0) {
-                flb_hash_add(ht, lookup_key->key, lookup_key->key_len,
-                             (void *) val->via.str.ptr, val->via.str.size);
+        
+        flb_config_map_foreach(head, lookup_key, ctx->lookup_keys) {
+            if (strncasecmp(key->via.str.ptr, lookup_key->val.str, 
+                flb_sds_len(lookup_key->val.str)) == 0) {
+                flb_hash_table_add(ht, lookup_key->val.str, flb_sds_len(lookup_key->val.str),
+                                   (void *) val->via.str.ptr, val->via.str.size);
             }
         }
     }
@@ -193,9 +194,9 @@ static MMDB_lookup_result_s mmdb_lookup(struct geoip2_ctx *ctx, const char *ip)
 }
 
 static void add_geoip_fields(msgpack_object *map,
-                             struct flb_hash *lookup_keys,
+                             struct flb_hash_table *lookup_keys,
                              struct geoip2_ctx *ctx,
-                             msgpack_packer *packer)
+                             struct flb_log_event_encoder *encoder)
 {
     int ret;
     struct mk_list *head;
@@ -220,19 +221,19 @@ static void add_geoip_fields(msgpack_object *map,
     mk_list_foreach_safe(head, tmp, &ctx->records) {
         record = mk_list_entry(head, struct geoip2_record, _head);
 
-        msgpack_pack_str(packer, record->key_len);
-        msgpack_pack_str_body(packer, record->key, record->key_len);
+        flb_log_event_encoder_append_body_string(
+            encoder, record->key, record->key_len);
 
-        ret = flb_hash_get(lookup_keys, record->lookup_key, record->lookup_key_len,
-                           (void *) &ip, &ip_size);
+        ret = flb_hash_table_get(lookup_keys, record->lookup_key, record->lookup_key_len,
+                                 (void *) &ip, &ip_size);
         if (ret == -1) {
-            msgpack_pack_nil(packer);
+            flb_log_event_encoder_append_body_null(encoder);
             continue;
         }
 
         result = mmdb_lookup(ctx, ip);
         if (!result.found_entry) {
-            msgpack_pack_nil(packer);
+            flb_log_event_encoder_append_body_null(encoder);
             continue;
         }
         entry = result.entry;
@@ -257,60 +258,65 @@ static void add_geoip_fields(msgpack_object *map,
         flb_free(path);
         if (status != MMDB_SUCCESS) {
             flb_plg_warn(ctx->ins, "cannot get value: %s", MMDB_strerror(status));
-            msgpack_pack_nil(packer);
+            flb_log_event_encoder_append_body_null(encoder);
             continue;
         }
         if (!entry_data.has_data) {
             flb_plg_warn(ctx->ins, "found entry does not have data");
-            msgpack_pack_nil(packer);
+            flb_log_event_encoder_append_body_null(encoder);
             continue;
         }
         if (entry_data.type == MMDB_DATA_TYPE_MAP ||
             entry_data.type == MMDB_DATA_TYPE_ARRAY) {
             flb_plg_warn(ctx->ins, "Not supported MAP and ARRAY");
-            msgpack_pack_nil(packer);
+            flb_log_event_encoder_append_body_null(encoder);
             continue;
         }
 
         switch (entry_data.type) {
         case MMDB_DATA_TYPE_EXTENDED:
             /* TODO: not implemented */
-            msgpack_pack_nil(packer);
+            flb_log_event_encoder_append_body_null(encoder);
             break;
         case MMDB_DATA_TYPE_POINTER:
             /* TODO: not implemented */
-            msgpack_pack_nil(packer);
+            flb_log_event_encoder_append_body_null(encoder);
             break;
         case MMDB_DATA_TYPE_UTF8_STRING:
-            msgpack_pack_str(packer, entry_data.data_size);
-            msgpack_pack_str_body(packer,
-                                  entry_data.utf8_string,
-                                  entry_data.data_size);
+            flb_log_event_encoder_append_body_string(
+                encoder,
+                (char *) entry_data.utf8_string,
+                entry_data.data_size);
             break;
         case MMDB_DATA_TYPE_DOUBLE:
-            msgpack_pack_double(packer, entry_data.double_value);
+            flb_log_event_encoder_append_body_double(
+                encoder, entry_data.double_value);
             break;
         case MMDB_DATA_TYPE_BYTES:
-            msgpack_pack_str(packer, entry_data.data_size);
-            msgpack_pack_str_body(packer,
-                                  entry_data.bytes,
-                                  entry_data.data_size);
+            flb_log_event_encoder_append_body_string(
+                encoder,
+                (char *) entry_data.bytes,
+                entry_data.data_size);
             break;
         case MMDB_DATA_TYPE_UINT16:
-            msgpack_pack_uint16(packer, entry_data.uint16);
+            flb_log_event_encoder_append_body_uint16(
+                encoder, entry_data.uint16);
             break;
         case MMDB_DATA_TYPE_UINT32:
-            msgpack_pack_uint32(packer, entry_data.uint32);
+            flb_log_event_encoder_append_body_uint32(
+                encoder, entry_data.uint32);
             break;
         case MMDB_DATA_TYPE_MAP:
             /* TODO: not implemented */
-            msgpack_pack_nil(packer);
+            flb_log_event_encoder_append_body_null(encoder);
             break;
         case MMDB_DATA_TYPE_INT32:
-            msgpack_pack_int32(packer, entry_data.int32);
+            flb_log_event_encoder_append_body_int32(
+                encoder, entry_data.int32);
             break;
         case MMDB_DATA_TYPE_UINT64:
-            msgpack_pack_uint64(packer, entry_data.uint64);
+            flb_log_event_encoder_append_body_uint64(
+                encoder, entry_data.uint64);
             break;
         case MMDB_DATA_TYPE_UINT128:
 #if !(MMDB_UINT128_IS_BYTE_ARRAY)
@@ -319,29 +325,30 @@ static void add_geoip_fields(msgpack_object *map,
 #else
             flb_warn("Not implemented when MMDB_UINT128_IS_BYTE_ARRAY");
 #endif
-            msgpack_pack_nil(packer);
+            flb_log_event_encoder_append_body_null(encoder);
             break;
         case MMDB_DATA_TYPE_ARRAY:
             /* TODO: not implemented */
-            msgpack_pack_nil(packer);
+            flb_log_event_encoder_append_body_null(encoder);
             break;
         case MMDB_DATA_TYPE_CONTAINER:
             /* TODO: not implemented */
-            msgpack_pack_nil(packer);
+            flb_log_event_encoder_append_body_null(encoder);
             break;
         case MMDB_DATA_TYPE_END_MARKER:
             break;
         case MMDB_DATA_TYPE_BOOLEAN:
-            entry_data.boolean ? msgpack_pack_true(packer) : msgpack_pack_false(packer);
+            flb_log_event_encoder_append_body_boolean(
+                encoder, (int) entry_data.boolean);
             break;
         case MMDB_DATA_TYPE_FLOAT:
-            msgpack_pack_float(packer, entry_data.float_value);
+            flb_log_event_encoder_append_body_double(
+                encoder, entry_data.float_value);
             break;
         default:
             flb_error("Unknown type: %d", entry_data.type);
             break;
         }
-
     }
 }
 
@@ -351,12 +358,11 @@ static int cb_geoip2_init(struct flb_filter_instance *f_ins,
 {
     struct geoip2_ctx *ctx = NULL;
     /* Create context */
-    ctx = flb_malloc(sizeof(struct geoip2_ctx));
+    ctx = flb_calloc(1, sizeof(struct geoip2_ctx));
     if (!ctx) {
         flb_errno();
         return -1;
     }
-    mk_list_init(&ctx->lookup_keys);
     mk_list_init(&ctx->records);
 
 
@@ -379,58 +385,93 @@ static int cb_geoip2_filter(const void *data, size_t bytes,
                             void *context,
                             struct flb_config *config)
 {
-    (void) i_ins;
     struct geoip2_ctx *ctx = context;
-    size_t off = 0;
-    int map_num = 0;
-    struct flb_time tm;
-    msgpack_sbuffer sbuffer;
-    msgpack_packer packer;
-    msgpack_unpacked unpacked;
-    msgpack_object *obj;
     msgpack_object_kv *kv;
-    struct flb_hash *lookup_keys_hash;
+    struct flb_hash_table *lookup_keys_hash;
+    struct flb_log_event_encoder log_encoder;
+    struct flb_log_event_decoder log_decoder;
+    struct flb_log_event log_event;
+    int ret;
+    int i;
 
-    /* Create temporal msgpack buffer */
-    msgpack_sbuffer_init(&sbuffer);
-    msgpack_packer_init(&packer, &sbuffer, msgpack_sbuffer_write);
+    (void) i_ins;
 
-    /* Iterate each item to know map number */
-    msgpack_unpacked_init(&unpacked);
-    while (msgpack_unpack_next(&unpacked, data, bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
-        if (unpacked.data.type != MSGPACK_OBJECT_ARRAY) {
-            continue;
-        }
+    ret = flb_log_event_decoder_init(&log_decoder, (char *) data, bytes);
 
-        flb_time_pop_from_msgpack(&tm, &unpacked, &obj);
+    if (ret != FLB_EVENT_DECODER_SUCCESS) {
+        flb_plg_error(ctx->ins,
+                      "Log event decoder initialization error : %d", ret);
 
-        if (obj->type == MSGPACK_OBJECT_MAP) {
-            map_num = obj->via.map.size;
-        }
-        else {
-            continue;
-        }
-
-        msgpack_pack_array(&packer, 2);
-        flb_time_append_to_msgpack(&tm, &packer, 0);
-
-        msgpack_pack_map(&packer, map_num + ctx->records_num);
-        kv = obj->via.map.ptr;
-        for (int i = 0; i < map_num; i++) {
-            msgpack_pack_object(&packer, (kv + i)->key);
-            msgpack_pack_object(&packer, (kv + i)->val);
-        }
-
-        lookup_keys_hash = prepare_lookup_keys(obj, ctx);
-        add_geoip_fields(obj, lookup_keys_hash, ctx, &packer);
-        flb_hash_destroy(lookup_keys_hash);
+        return FLB_FILTER_NOTOUCH;
     }
-    msgpack_unpacked_destroy(&unpacked);
 
-    /* link new buffers */
-    *out_buf = sbuffer.data;
-    *out_size = sbuffer.size;
-    return FLB_FILTER_MODIFIED;
+    ret = flb_log_event_encoder_init(&log_encoder,
+                                     FLB_LOG_EVENT_FORMAT_DEFAULT);
+
+    if (ret != FLB_EVENT_ENCODER_SUCCESS) {
+        flb_plg_error(ctx->ins,
+                      "Log event encoder initialization error : %d", ret);
+
+        flb_log_event_decoder_destroy(&log_decoder);
+
+        return FLB_FILTER_NOTOUCH;
+    }
+
+    while ((ret = flb_log_event_decoder_next(
+                    &log_decoder,
+                    &log_event)) == FLB_EVENT_DECODER_SUCCESS) {
+
+        ret = flb_log_event_encoder_begin_record(&log_encoder);
+
+        if (ret == FLB_EVENT_ENCODER_SUCCESS) {
+            ret = flb_log_event_encoder_set_timestamp(
+                    &log_encoder, &log_event.timestamp);
+        }
+
+        kv = log_event.body->via.map.ptr;
+        for (i = 0;
+             i < log_event.body->via.map.size &&
+             ret == FLB_EVENT_ENCODER_SUCCESS ;
+             i++) {
+            ret = flb_log_event_encoder_append_body_values(
+                    &log_encoder,
+                    FLB_LOG_EVENT_MSGPACK_OBJECT_VALUE(&kv[i].key),
+                    FLB_LOG_EVENT_MSGPACK_OBJECT_VALUE(&kv[i].val));
+        }
+
+        lookup_keys_hash = prepare_lookup_keys(log_event.body, ctx);
+        add_geoip_fields(log_event.body, lookup_keys_hash, ctx, &log_encoder);
+        flb_hash_table_destroy(lookup_keys_hash);
+
+        if (ret == FLB_EVENT_ENCODER_SUCCESS) {
+            ret = flb_log_event_encoder_commit_record(&log_encoder);
+        }
+    }
+
+    if (ret == FLB_EVENT_DECODER_ERROR_INSUFFICIENT_DATA &&
+        log_decoder.offset == bytes) {
+        ret = FLB_EVENT_ENCODER_SUCCESS;
+    }
+
+    if (ret == FLB_EVENT_ENCODER_SUCCESS) {
+        *out_buf  = log_encoder.output_buffer;
+        *out_size = log_encoder.output_length;
+
+        ret = FLB_FILTER_MODIFIED;
+
+        flb_log_event_encoder_claim_internal_buffer_ownership(&log_encoder);
+    }
+    else {
+        flb_plg_error(ctx->ins,
+                      "Log event encoder error : %d", ret);
+
+        ret = FLB_FILTER_NOTOUCH;
+    }
+
+    flb_log_event_decoder_destroy(&log_decoder);
+    flb_log_event_encoder_destroy(&log_encoder);
+
+    return ret;
 }
 
 static int cb_geoip2_exit(void *data, struct flb_config *config)
@@ -447,11 +488,32 @@ static int cb_geoip2_exit(void *data, struct flb_config *config)
     return 0;
 }
 
+static struct flb_config_map config_map[] = {
+    {
+     FLB_CONFIG_MAP_STR, "database", (char *)NULL,
+     0, FLB_TRUE, offsetof(struct geoip2_ctx, database),
+     "Set the geoip2 database path"
+    },
+    {
+     FLB_CONFIG_MAP_STR, "lookup_key", NULL,
+     FLB_CONFIG_MAP_MULT, FLB_TRUE, offsetof(struct geoip2_ctx, lookup_keys),
+     "Add a lookup_key"
+    },
+    {
+     FLB_CONFIG_MAP_STR, "record", NULL,
+     FLB_CONFIG_MAP_MULT, FLB_TRUE, offsetof(struct geoip2_ctx, record_keys),
+     "Add a record to the output base on geoip2"
+    },
+    /* EOF */
+    {0}
+};
+
 struct flb_filter_plugin filter_geoip2_plugin = {
-    .name = "geoip2",
+    .name        = "geoip2",
     .description = "add geoip information to records",
-    .cb_init = cb_geoip2_init,
-    .cb_filter = cb_geoip2_filter,
-    .cb_exit = cb_geoip2_exit,
-    .flags = 0,
+    .cb_init     = cb_geoip2_init,
+    .cb_filter   = cb_geoip2_filter,
+    .cb_exit     = cb_geoip2_exit,
+    .config_map  = config_map,
+    .flags       = 0,
 };
