@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -23,23 +22,25 @@
 #include <stdint.h>
 #include <fluent-bit/flb_config.h>
 #include <fluent-bit/flb_input.h>
+#include <fluent-bit/flb_log_event_encoder.h>
 
-#define DEFAULT_INTERVAL_SEC  1
-#define DEFAULT_INTERVAL_NSEC 0
+#define DEFAULT_INTERVAL_SEC  "1"
+#define DEFAULT_INTERVAL_NSEC "0"
 
 #define STR_KEY_WRITE "write_size"
 #define STR_KEY_READ  "read_size"
 
 struct flb_in_disk_config {
-    uint64_t *read_total;
-    uint64_t *write_total;
-    uint64_t *prev_read_total;
-    uint64_t *prev_write_total;
-    char     *dev_name;
-    int      entry;
-    int      interval_sec;
-    int      interval_nsec;
-    int      first_snapshot;   /* a feild to indicate whethor or not this is the first collect*/
+    uint64_t  *read_total;
+    uint64_t  *write_total;
+    uint64_t  *prev_read_total;
+    uint64_t  *prev_write_total;
+    flb_sds_t dev_name;
+    int       entry;
+    int       interval_sec;
+    int       interval_nsec;
+    int       first_snapshot;   /* a feild to indicate whethor or not this is the first collect*/
+    struct flb_log_event_encoder log_encoder;
 };
 
 extern struct flb_input_plugin in_disk_plugin;
