@@ -47,8 +47,8 @@ struct flb_dummy {
     int  start_time_sec;
     int  start_time_nsec;
 
-    bool fixed_timestamp;
-    bool flush_on_startup;
+    int fixed_timestamp;
+    int flush_on_startup;
 
     char *ref_metadata_msgpack;
     size_t ref_metadata_msgpack_size;
