@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -23,6 +22,7 @@
 
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_input.h>
+#include <fluent-bit/flb_log_event_encoder.h>
 
 #include <stdint.h>
 
@@ -49,6 +49,8 @@ struct flb_in_kmsg_config {
     int fd;                    /* descriptor -> FLB_KMSG_DEV */
     struct timeval boot_time;  /* System boot time           */
 
+    int prio_level;
+
     /* Line processing */
     int buffer_id;
 
@@ -56,6 +58,7 @@ struct flb_in_kmsg_config {
     char *buf_data;
     size_t buf_len;
     size_t buf_size;
+    struct flb_log_event_encoder log_encoder;
     struct flb_input_instance *ins;
 };
 
