@@ -741,6 +741,7 @@ int splunk_prot_handle(struct flb_splunk *ctx, struct splunk_conn *conn,
     if (ctx->ins->tag && !ctx->ins->tag_default) {
         tag = flb_sds_create(ctx->ins->tag);
         if (tag == NULL) {
+            mk_mem_free(uri);
             return -1;
         }
     }
@@ -833,7 +834,8 @@ int splunk_prot_handle(struct flb_splunk *ctx, struct splunk_conn *conn,
     flb_log_event_encoder_reset(&ctx->log_encoder);
 
     if (request->method == MK_METHOD_POST) {
-        if (strcasecmp(uri, "/services/collector/raw") == 0) {
+        if (strcasecmp(uri, "/services/collector/raw/1.0") == 0 ||
+            strcasecmp(uri, "/services/collector/raw") == 0) {
             ret = process_hec_raw_payload(ctx, conn, tag, session, request);
 
             if (!ret) {
@@ -841,7 +843,8 @@ int splunk_prot_handle(struct flb_splunk *ctx, struct splunk_conn *conn,
             }
             send_json_message_response(conn, 200, "{\"text\":\"Success\",\"code\":0}");
         }
-        else if (strcasecmp(uri, "/services/collector/event") == 0 ||
+        else if (strcasecmp(uri, "/services/collector/event/1.0") == 0 ||
+                 strcasecmp(uri, "/services/collector/event") == 0 ||
                  strcasecmp(uri, "/services/collector") == 0) {
             ret = process_hec_payload(ctx, conn, tag, session, request);
 
@@ -901,8 +904,8 @@ int splunk_prot_handle_error(struct flb_splunk *ctx, struct splunk_conn *conn,
 
 /* New gen HTTP server */
 
-static int send_response_ng(struct flb_http_response *response, 
-                            int http_status, 
+static int send_response_ng(struct flb_http_response *response,
+                            int http_status,
                             char *message)
 {
     flb_http_response_set_status(response, http_status);
@@ -921,8 +924,8 @@ static int send_response_ng(struct flb_http_response *response,
     }
 
     if (message != NULL) {
-        flb_http_response_set_body(response, 
-                                   (unsigned char *) message, 
+        flb_http_response_set_body(response,
+                                   (unsigned char *) message,
                                    strlen(message));
     }
 
@@ -931,8 +934,8 @@ static int send_response_ng(struct flb_http_response *response,
     return 0;
 }
 
-static int send_json_message_response_ng(struct flb_http_response *response, 
-                                         int http_status, 
+static int send_json_message_response_ng(struct flb_http_response *response,
+                                         int http_status,
                                          char *message)
 {
     flb_http_response_set_status(response, http_status);
@@ -950,13 +953,13 @@ static int send_json_message_response_ng(struct flb_http_response *response,
         flb_http_response_set_message(response, "Bad Request");
     }
 
-    flb_http_response_set_header(response, 
+    flb_http_response_set_header(response,
                                 "content-type", 0,
                                 "application/json", 0);
 
     if (message != NULL) {
-        flb_http_response_set_body(response, 
-                                   (unsigned char *) message, 
+        flb_http_response_set_body(response,
+                                   (unsigned char *) message,
                                    strlen(message));
     }
 
@@ -1099,7 +1102,7 @@ int splunk_prot_handle_ng(struct flb_http_request *request,
     }
 
     /* HTTP/1.1 needs Host header */
-    if (request->protocol_version == HTTP_PROTOCOL_HTTP1 && 
+    if (request->protocol_version == HTTP_PROTOCOL_HTTP1 &&
         request->host == NULL) {
 
         return -1;
@@ -1140,7 +1143,7 @@ int splunk_prot_handle_ng(struct flb_http_request *request,
     if (request->method != HTTP_METHOD_POST) {
         /* HEAD, PUT, PATCH, and DELETE methods are prohibited to use.*/
         send_response_ng(response, 400, "error: invalid HTTP method\n");
-        
+
         return -1;
     }
 
@@ -1150,7 +1153,8 @@ int splunk_prot_handle_ng(struct flb_http_request *request,
         return -1;
     }
 
-    if (strcasecmp(request->path, "/services/collector/raw") == 0) {
+    if (strcasecmp(request->path, "/services/collector/raw/1.0") == 0 ||
+        strcasecmp(request->path, "/services/collector/raw") == 0) {
         ret = process_hec_raw_payload_ng(request, response, tag, context);
 
         if (ret != 0) {
@@ -1162,7 +1166,8 @@ int splunk_prot_handle_ng(struct flb_http_request *request,
 
         ret = 0;
     }
-    else if (strcasecmp(request->path, "/services/collector/event") == 0 ||
+    else if (strcasecmp(request->path, "/services/collector/event/1.0") == 0 ||
+             strcasecmp(request->path, "/services/collector/event") == 0 ||
              strcasecmp(request->path, "/services/collector") == 0) {
         ret = process_hec_payload_ng(request, response, tag, context);
 
