@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -23,13 +22,12 @@
 
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_config.h>
+#include <fluent-bit/flb_connection.h>
 
 #include "syslog.h"
 
 /* Respresents a connection */
 struct syslog_conn {
-    struct mk_event event;           /* Built-in event data for mk_events */
-    int fd;                          /* Socket file descriptor            */
     int status;                      /* Connection status                 */
 
     /* Buffer */
@@ -39,12 +37,16 @@ struct syslog_conn {
     size_t buf_parsed;               /* Parsed buffer (offset)            */
     struct flb_input_instance *ins;  /* Parent plugin instance            */
     struct flb_syslog *ctx;          /* Plugin configuration context      */
+    struct flb_connection *connection;
 
     struct mk_list _head;
 };
 
 int syslog_conn_event(void *data);
-struct syslog_conn *syslog_conn_add(int fd, struct flb_syslog *ctx);
+int syslog_stream_conn_event(void *data);
+int syslog_dgram_conn_event(void *data);
+struct syslog_conn *syslog_conn_add(struct flb_connection *connection,
+                                    struct flb_syslog *ctx);
 int syslog_conn_del(struct syslog_conn *conn);
 int syslog_conn_exit(struct flb_syslog *ctx);
 
