@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -25,7 +24,6 @@
 #include <fluent-bit/flb_upstream.h>
 #include <fluent-bit/flb_env.h>
 #include <fluent-bit/flb_fstore.h>
-#include <mbedtls/sha256.h>
 
 /* End point */
 #define CALYPTIA_HOST            "cloud-api.calyptia.com"
@@ -35,11 +33,13 @@
 #define CALYPTIA_ACTION_REGISTER  0
 #define CALYPTIA_ACTION_PATCH     1
 #define CALYPTIA_ACTION_METRICS   2
+#define CALYPTIA_ACTION_TRACE     3
 
 /* Endpoints */
 #define CALYPTIA_ENDPOINT_CREATE  "/v1/agents"
 #define CALYPTIA_ENDPOINT_PATCH   "/v1/agents/%s"
 #define CALYPTIA_ENDPOINT_METRICS "/v1/agents/%s/metrics"
+#define CALYPTIA_ENDPOINT_TRACE   "/v1/traces/%s"
 
 /* Storage */
 #define CALYPTIA_SESSION_FILE     "session.CALYPTIA"
@@ -65,6 +65,7 @@ struct flb_calyptia {
     flb_sds_t agent_id;
     flb_sds_t agent_token;
     flb_sds_t machine_id;                 /* machine-id  */
+    flb_sds_t fleet_id;                   /* fleet-id  */
     flb_sds_t metrics_endpoint;           /* metrics endpoint */
     struct flb_fstore *fs;                /* fstore ctx */
     struct flb_fstore_stream *fs_stream;  /* fstore stream */
@@ -74,6 +75,11 @@ struct flb_calyptia {
     struct mk_list kv_labels;             /* parsed add_labels */
     struct flb_output_instance *ins;      /* plugin instance */
     struct flb_config *config;            /* Fluent Bit context */
+/* used for reporting chunk trace records to calyptia cloud. */
+#ifdef FLB_HAVE_CHUNK_TRACE
+    flb_sds_t trace_endpoint;
+    flb_sds_t pipeline_id;
+#endif /* FLB_HAVE_CHUNK_TRACE */
 };
 
-#endif
\ No newline at end of file
+#endif
