@@ -120,9 +120,9 @@ static int datadog_format(struct flb_config *config,
     /* in normal flush callback we have the event_chunk set as flush context
      * so we don't need to calculate the event len.
      * But in test mode the formatter won't get the event_chunk as flush_ctx
-     */ 
+     */
     if (flush_ctx != NULL) {
-        event_chunk = flush_ctx; 
+        event_chunk = flush_ctx;
         array_size = event_chunk->total_events;
     } else {
         array_size = flb_mp_count(data, bytes);
@@ -292,25 +292,24 @@ static int datadog_format(struct flb_config *config,
         /* here we concatenate ctx->dd_tags and remapped_tags, depending on their presence */
         if (remap_cnt) {
             if (ctx->dd_tags != NULL) {
-                tmp = flb_sds_cat(remapped_tags, FLB_DATADOG_TAG_SEPERATOR,
-                                  strlen(FLB_DATADOG_TAG_SEPERATOR));
-                if (!tmp) {
+                ret = flb_sds_cat_safe(&remapped_tags, FLB_DATADOG_TAG_SEPERATOR,
+                                       strlen(FLB_DATADOG_TAG_SEPERATOR));
+                if (ret < 0) {
                     flb_errno();
                     flb_sds_destroy(remapped_tags);
                     msgpack_sbuffer_destroy(&mp_sbuf);
                     flb_log_event_decoder_destroy(&log_decoder);
                     return -1;
                 }
-                remapped_tags = tmp;
-                flb_sds_cat(remapped_tags, ctx->dd_tags, strlen(ctx->dd_tags));
-                if (!tmp) {
+
+                ret = flb_sds_cat_safe(&remapped_tags, ctx->dd_tags, strlen(ctx->dd_tags));
+                if (ret < 0) {
                     flb_errno();
                     flb_sds_destroy(remapped_tags);
                     msgpack_sbuffer_destroy(&mp_sbuf);
                     flb_log_event_decoder_destroy(&log_decoder);
                     return -1;
                 }
-                remapped_tags = tmp;
             }
             dd_msgpack_pack_key_value_str(&mp_pck,
                                           FLB_DATADOG_DD_TAGS_KEY,
