@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -25,6 +24,7 @@
 #include <fluent-bit/flb_sds.h>
 #include <fluent-bit/flb_http_client.h>
 #include <fluent-bit/flb_config_map.h>
+#include <fluent-bit/flb_log_event_decoder.h>
 #include <msgpack.h>
 
 #include "kafka.h"
@@ -78,6 +78,11 @@ static struct flb_config_map config_map[] = {
      0, FLB_TRUE, offsetof(struct flb_kafka_rest, tag_key),
      "Specify the key name of the record if include_tag_key is enabled. "
     },
+    {
+     FLB_CONFIG_MAP_BOOL, "avro_http_header", "false",
+     0, FLB_TRUE, offsetof(struct flb_kafka_rest, avro_http_header),
+     "Specify if the format has avro header in http request"
+    },
 
     /* EOF */
     {0}
@@ -96,33 +101,33 @@ static flb_sds_t kafka_rest_format(const void *data, size_t bytes,
     int arr_size = 0;
     int map_size;
     size_t s;
-    size_t off = 0;
     flb_sds_t out_buf;
     char time_formatted[256];
-    msgpack_unpacked result;
-    msgpack_object root;
     msgpack_object map;
-    msgpack_object *obj;
     msgpack_object key;
     msgpack_object val;
     struct tm tm;
-    struct flb_time tms;
     msgpack_sbuffer mp_sbuf;
     msgpack_packer mp_pck;
+    struct flb_log_event_decoder log_decoder;
+    struct flb_log_event log_event;
+    int ret;
+
+    ret = flb_log_event_decoder_init(&log_decoder, (char *) data, bytes);
+
+    if (ret != FLB_EVENT_DECODER_SUCCESS) {
+        flb_plg_error(ctx->ins,
+                      "Log event decoder initialization error : %d", ret);
+
+        return NULL;
+    }
 
     /* Init temporary buffers */
     msgpack_sbuffer_init(&mp_sbuf);
     msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
 
-    /* Iterate the original buffer and perform adjustments */
-    msgpack_unpacked_init(&result);
-
     /* Count number of entries */
-    while (msgpack_unpack_next(&result, data, bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
-        arr_size++;
-    }
-    msgpack_unpacked_destroy(&result);
-    off = 0;
+    arr_size = flb_mp_count(data, bytes);
 
     /* Root map */
     msgpack_pack_map(&mp_pck, 1);
@@ -132,15 +137,12 @@ static flb_sds_t kafka_rest_format(const void *data, size_t bytes,
     msgpack_pack_array(&mp_pck, arr_size);
 
     /* Iterate and compose array content */
-    msgpack_unpacked_init(&result);
-    while (msgpack_unpack_next(&result, data, bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
-        root = result.data;
-
-        map = root.via.array.ptr[1];
+    while ((ret = flb_log_event_decoder_next(
+                    &log_decoder,
+                    &log_event)) == FLB_EVENT_DECODER_SUCCESS) {
+        map = *log_event.body;
         map_size = 1;
 
-        flb_time_pop_from_msgpack(&tms, &result, &obj);
-
         if (ctx->partition >= 0) {
             map_size++;
         }
@@ -181,11 +183,11 @@ static flb_sds_t kafka_rest_format(const void *data, size_t bytes,
         msgpack_pack_str_body(&mp_pck, ctx->time_key, ctx->time_key_len);
 
         /* Format the time */
-        gmtime_r(&tms.tm.tv_sec, &tm);
+        gmtime_r(&log_event.timestamp.tm.tv_sec, &tm);
         s = strftime(time_formatted, sizeof(time_formatted) - 1,
                      ctx->time_key_format, &tm);
         len = snprintf(time_formatted + s, sizeof(time_formatted) - 1 - s,
-                       ".%09" PRIu64 "Z", (uint64_t) tms.tm.tv_nsec);
+                       ".%09" PRIu64 "Z", (uint64_t) log_event.timestamp.tm.tv_nsec);
         s += len;
         msgpack_pack_str(&mp_pck, s);
         msgpack_pack_str_body(&mp_pck, time_formatted, s);
@@ -206,7 +208,7 @@ static flb_sds_t kafka_rest_format(const void *data, size_t bytes,
             msgpack_pack_object(&mp_pck, val);
         }
     }
-    msgpack_unpacked_destroy(&result);
+    flb_log_event_decoder_destroy(&log_decoder);
 
     /* Convert to JSON */
     out_buf = flb_msgpack_raw_to_json_sds(mp_sbuf.data, mp_sbuf.size);
@@ -216,6 +218,7 @@ static flb_sds_t kafka_rest_format(const void *data, size_t bytes,
     }
 
     *out_size = flb_sds_len(out_buf);
+
     return out_buf;
 }
 
@@ -241,8 +244,8 @@ static int cb_kafka_init(struct flb_output_instance *ins,
     return 0;
 }
 
-static void cb_kafka_flush(const void *data, size_t bytes,
-                           const char *tag, int tag_len,
+static void cb_kafka_flush(struct flb_event_chunk *event_chunk,
+                           struct flb_output_flush *out_flush,
                            struct flb_input_instance *i_ins,
                            void *out_context,
                            struct flb_config *config)
@@ -252,11 +255,9 @@ static void cb_kafka_flush(const void *data, size_t bytes,
     size_t js_size;
     size_t b_sent;
     struct flb_http_client *c;
-    struct flb_upstream_conn *u_conn;
+    struct flb_connection *u_conn;
     struct flb_kafka_rest *ctx = out_context;
     (void) i_ins;
-    (void) tag;
-    (void) tag_len;
 
     /* Get upstream connection */
     u_conn = flb_upstream_conn_get(ctx->u);
@@ -265,7 +266,9 @@ static void cb_kafka_flush(const void *data, size_t bytes,
     }
 
     /* Convert format */
-    js = kafka_rest_format(data, bytes, tag, tag_len, &js_size, ctx);
+    js = kafka_rest_format(event_chunk->data, event_chunk->size,
+                           event_chunk->tag, flb_sds_len(event_chunk->tag),
+                           &js_size, ctx);
     if (!js) {
         flb_upstream_conn_release(u_conn);
         FLB_OUTPUT_RETURN(FLB_ERROR);
@@ -275,10 +278,17 @@ static void cb_kafka_flush(const void *data, size_t bytes,
     c = flb_http_client(u_conn, FLB_HTTP_POST, ctx->uri,
                         js, js_size, NULL, 0, NULL, 0);
     flb_http_add_header(c, "User-Agent", 10, "Fluent-Bit", 10);
-    flb_http_add_header(c,
-                        "Content-Type", 12,
-                        "application/vnd.kafka.json.v2+json", 34);
-
+    if (ctx->avro_http_header == FLB_TRUE) {
+        flb_http_add_header(c,
+                            "Content-Type", 12,
+                            "application/vnd.kafka.avro.v2+json", 34);
+    }
+    else {
+        flb_http_add_header(c,
+                            "Content-Type", 12,
+                            "application/vnd.kafka.json.v2+json", 34);
+    }
+    
     if (ctx->http_user && ctx->http_passwd) {
         flb_http_basic_auth(c, ctx->http_user, ctx->http_passwd);
     }
