@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -22,6 +21,8 @@
 #include <fluent-bit/flb_utils.h>
 #include <fluent-bit/flb_pack.h>
 #include <fluent-bit/flb_time.h>
+#include <fluent-bit/flb_config_map.h>
+#include <fluent-bit/flb_log_event_decoder.h>
 
 #include <stdio.h>
 #include <msgpack.h>
@@ -32,6 +33,7 @@ static int cb_nats_init(struct flb_output_instance *ins, struct flb_config *conf
                         void *data)
 {
     int io_flags;
+    int ret;
     struct flb_upstream *upstream;
     struct flb_out_nats_config *ctx;
 
@@ -45,6 +47,14 @@ static int cb_nats_init(struct flb_output_instance *ins, struct flb_config *conf
         return -1;
     }
 
+    /* Set default values */
+    ret = flb_output_config_map_set(ins, ctx);
+    if (ret == -1) {
+        flb_plg_error(ins, "flb_output_config_map_set failed");
+        flb_free(ctx);
+        return -1;
+    }
+
     io_flags = FLB_IO_TCP;
     if (ins->host.ipv6 == FLB_TRUE) {
         io_flags |= FLB_IO_IPV6;
@@ -68,51 +78,48 @@ static int cb_nats_init(struct flb_output_instance *ins, struct flb_config *conf
     return 0;
 }
 
-static int msgpack_to_json(const void *data, size_t bytes,
+static int msgpack_to_json(struct flb_out_nats_config *ctx,
+                           const void *data, size_t bytes,
                            const char *tag, int tag_len,
                            char **out_json, size_t *out_size)
 {
     int i;
     int map_size;
-    size_t off = 0;
     size_t array_size = 0;
     flb_sds_t out_buf;
     msgpack_object map;
-    msgpack_object root;
     msgpack_object m_key;
     msgpack_object m_val;
-    msgpack_packer   mp_pck;
-    msgpack_sbuffer  mp_sbuf;
-    msgpack_unpacked result;
-    msgpack_object *obj;
-    struct flb_time tm;
+    msgpack_packer mp_pck;
+    msgpack_sbuffer mp_sbuf;
+    struct flb_log_event_decoder log_decoder;
+    struct flb_log_event log_event;
+    int ret;
 
-    /* Iterate the original buffer and perform adjustments */
-    msgpack_unpacked_init(&result);
-    while (msgpack_unpack_next(&result, data, bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
-        array_size++;
+    ret = flb_log_event_decoder_init(&log_decoder, (char *) data, bytes);
+
+    if (ret != FLB_EVENT_DECODER_SUCCESS) {
+        flb_plg_error(ctx->ins,
+                      "Log event decoder initialization error : %d", ret);
+
+        return -1;
     }
-    msgpack_unpacked_destroy(&result);
-    msgpack_unpacked_init(&result);
-    off = 0;
+
+    array_size = flb_mp_count(data, bytes);
 
     /* Convert MsgPack to JSON */
     msgpack_sbuffer_init(&mp_sbuf);
     msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
     msgpack_pack_array(&mp_pck, array_size);
 
-    while (msgpack_unpack_next(&result, data, bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
-        if (result.data.type != MSGPACK_OBJECT_ARRAY) {
-            continue;
-        }
-        root = result.data;
-
-        flb_time_pop_from_msgpack(&tm, &result, &obj);
-        map    = root.via.array.ptr[1];
+    while ((ret = flb_log_event_decoder_next(
+                    &log_decoder,
+                    &log_event)) == FLB_EVENT_DECODER_SUCCESS) {
+        map      = *log_event.body;
         map_size = map.via.map.size;
 
         msgpack_pack_array(&mp_pck, 2);
-        msgpack_pack_double(&mp_pck, flb_time_to_double(&tm));
+        msgpack_pack_double(&mp_pck, flb_time_to_double(&log_event.timestamp));
 
         msgpack_pack_map(&mp_pck, map_size + 1);
         msgpack_pack_str(&mp_pck, 3);
@@ -128,7 +135,8 @@ static int msgpack_to_json(const void *data, size_t bytes,
             msgpack_pack_object(&mp_pck, m_val);
         }
     }
-    msgpack_unpacked_destroy(&result);
+
+    flb_log_event_decoder_destroy(&log_decoder);
 
     out_buf = flb_msgpack_raw_to_json_sds(mp_sbuf.data, mp_sbuf.size);
     msgpack_sbuffer_destroy(&mp_sbuf);
@@ -143,11 +151,11 @@ static int msgpack_to_json(const void *data, size_t bytes,
     return 0;
 }
 
-void cb_nats_flush(const void *data, size_t bytes,
-                   const char *tag, int tag_len,
-                   struct flb_input_instance *i_ins,
-                   void *out_context,
-                   struct flb_config *config)
+static void cb_nats_flush(struct flb_event_chunk *event_chunk,
+                          struct flb_output_flush *out_flush,
+                          struct flb_input_instance *i_ins,
+                          void *out_context,
+                          struct flb_config *config)
 {
     int ret;
     size_t bytes_sent;
@@ -156,7 +164,7 @@ void cb_nats_flush(const void *data, size_t bytes,
     char *request;
     int req_len;
     struct flb_out_nats_config *ctx = out_context;
-    struct flb_upstream_conn *u_conn;
+    struct flb_connection *u_conn;
 
     u_conn = flb_upstream_conn_get(ctx->u);
     if (!u_conn) {
@@ -175,14 +183,17 @@ void cb_nats_flush(const void *data, size_t bytes,
     }
 
     /* Convert original Fluent Bit MsgPack format to JSON */
-    ret = msgpack_to_json(data, bytes, tag, tag_len, &json_msg, &json_len);
+    ret = msgpack_to_json(ctx,
+                          event_chunk->data, event_chunk->size,
+                          event_chunk->tag, flb_sds_len(event_chunk->tag),
+                          &json_msg, &json_len);
     if (ret == -1) {
         flb_upstream_conn_release(u_conn);
         FLB_OUTPUT_RETURN(FLB_ERROR);
     }
 
     /* Compose the NATS Publish request */
-    request = flb_malloc(json_len + tag_len + 32);
+    request = flb_malloc(json_len + flb_sds_len(event_chunk->tag) + 32);
     if (!request) {
         flb_errno();
         flb_sds_destroy(json_msg);
@@ -190,8 +201,9 @@ void cb_nats_flush(const void *data, size_t bytes,
         FLB_OUTPUT_RETURN(FLB_RETRY);
     }
 
-    req_len = snprintf(request, tag_len + 32, "PUB %s %zu\r\n",
-                       tag, json_len);
+    req_len = snprintf(request, flb_sds_len(event_chunk->tag)+ 32,
+                       "PUB %s %zu\r\n",
+                       event_chunk->tag, json_len);
 
     /* Append JSON message and ending CRLF */
     memcpy(request + req_len, json_msg, json_len);
@@ -224,6 +236,11 @@ int cb_nats_exit(void *data, struct flb_config *config)
     return 0;
 }
 
+static struct flb_config_map config_map[] = {
+    /* EOF */
+    {0}
+};
+
 struct flb_output_plugin out_nats_plugin = {
     .name         = "nats",
     .description  = "NATS Server",
@@ -231,4 +248,5 @@ struct flb_output_plugin out_nats_plugin = {
     .cb_flush     = cb_nats_flush,
     .cb_exit      = cb_nats_exit,
     .flags        = FLB_OUTPUT_NET,
+    .config_map   = config_map
 };
