@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -126,11 +125,19 @@ struct s3_file *s3_store_file_get(struct flb_s3 *ctx, const char *tag,
 /* Append data to a new or existing fstore file */
 int s3_store_buffer_put(struct flb_s3 *ctx, struct s3_file *s3_file,
                         const char *tag, int tag_len,
-                        char *data, size_t bytes)
+                        char *data, size_t bytes,
+                        time_t file_first_log_time)
 {
     int ret;
     flb_sds_t name;
     struct flb_fstore_file *fsf;
+    size_t space_remaining;
+
+    if (ctx->store_dir_limit_size > 0 && ctx->current_buffer_size + bytes >= ctx->store_dir_limit_size) {
+        flb_plg_error(ctx->ins, "Buffer is full: current_buffer_size=%zu, new_data=%zu, store_dir_limit_size=%zu bytes",
+                    ctx->current_buffer_size, bytes, ctx->store_dir_limit_size);
+        return -1;
+    }
 
     /* If no target file was found, create a new one */
     if (!s3_file) {
@@ -169,6 +176,7 @@ int s3_store_buffer_put(struct flb_s3 *ctx, struct s3_file *s3_file,
             return -1;
         }
         s3_file->fsf = fsf;
+        s3_file->first_log_time = file_first_log_time;
         s3_file->create_time = time(NULL);
 
         /* Use fstore opaque 'data' reference to keep our context */
@@ -185,6 +193,17 @@ int s3_store_buffer_put(struct flb_s3 *ctx, struct s3_file *s3_file,
         return -1;
     }
     s3_file->size += bytes;
+    ctx->current_buffer_size += bytes;
+
+    /* if buffer is 95% full, warn user */
+    if (ctx->store_dir_limit_size > 0) {
+        space_remaining = ctx->store_dir_limit_size - ctx->current_buffer_size;
+        if ((space_remaining * 20) < ctx->store_dir_limit_size) {
+            flb_plg_warn(ctx->ins, "Buffer is almost full: current_buffer_size=%zu, store_dir_limit_size=%zu bytes",
+                        ctx->current_buffer_size, ctx->store_dir_limit_size);
+            return -1;
+        }
+    }
 
     return 0;
 }
@@ -224,6 +243,7 @@ static int set_files_context(struct flb_s3 *ctx)
                 continue;
             }
             s3_file->fsf = fsf;
+            s3_file->first_log_time = time(NULL);
             s3_file->create_time = time(NULL);
 
             /* Use fstore opaque 'data' reference to keep our context */
@@ -398,6 +418,7 @@ int s3_store_file_delete(struct flb_s3 *ctx, struct s3_file *s3_file)
     struct flb_fstore_file *fsf;
 
     fsf = s3_file->fsf;
+    ctx->current_buffer_size -= s3_file->size;
 
     /* permanent deletion */
     flb_fstore_file_delete(ctx->fs, fsf);
