@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -68,6 +67,16 @@ struct flb_out_tcp *flb_tcp_conf_create(struct flb_output_instance *ins,
         io_flags |= FLB_IO_IPV6;
     }
 
+    /* raw message key mode */
+    if (ctx->raw_message_key) {
+        ctx->ra_raw_message_key = flb_ra_create(ctx->raw_message_key, FLB_TRUE);
+        if (!ctx->ra_raw_message_key) {
+            flb_plg_error(ctx->ins, "could not create record accessor for raw_message_key");
+            flb_free(ctx);
+            return NULL;
+        }
+    }
+
     /* Upstream context */
     upstream = flb_upstream_create(config,
                                    ins->host.name,
@@ -132,6 +141,10 @@ void flb_tcp_conf_destroy(struct flb_out_tcp *ctx)
         return;
     }
 
+    if (ctx->ra_raw_message_key) {
+        flb_ra_destroy(ctx->ra_raw_message_key);
+    }
+
     if (ctx->u) {
         flb_upstream_destroy(ctx->u);
     }
