@@ -49,6 +49,10 @@ extern struct flb_aws_error_reporter *error_reporter;
 #ifdef FLB_SYSTEM_MACOS
 #include <CoreFoundation/CoreFoundation.h>
 #include <IOKit/IOKitLib.h>
+#include <AvailabilityMacros.h>
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 120000
+#define kIOMainPortDefault kIOMasterPortDefault
+#endif
 #endif
 
 /*
@@ -1403,7 +1407,7 @@ int flb_utils_get_machine_id(char **out_id, size_t *out_size)
     status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                           TEXT("SOFTWARE\\Microsoft\\Cryptography"),
                           0,
-                          KEY_QUERY_VALUE,
+                          KEY_QUERY_VALUE|KEY_WOW64_64KEY,
                           &hKey);
 
     if (status != ERROR_SUCCESS) {
@@ -1420,9 +1424,16 @@ int flb_utils_get_machine_id(char **out_id, size_t *out_size)
             return -1;
         }
 
-        *out_size = dwBufSize;
+        memcpy(*out_id, buf, dwBufSize);
+
+        /* RegQueryValueEx sets dwBufSize to strlen()+1 for the NULL 
+         * terminator, but we only need strlen(). */
+        *out_size = dwBufSize-1;
         return 0;
     }
+    else {
+        flb_error("unable to retrieve MachineGUID, error code: %d", status);
+    }
 #elif defined (FLB_SYSTEM_MACOS)
     bool bret;
     CFStringRef serialNumber;
@@ -1464,6 +1475,8 @@ int flb_utils_get_machine_id(char **out_id, size_t *out_size)
     }
 #endif
 
+    flb_warn("falling back on random machine UUID");
+
     /* generate a random uuid */
     uuid = flb_malloc(38);
     if (!uuid) {
